;; My Galaxy spaceship game
;; Copyright (C) 2017 AyMy Studios
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

.area _DATA

.include "headers/entity.h.s"
.include "headers/entity_inherited.h.s"

.globl _sprite_bala

new bullet 195, 0, 5, 6, _sprite_bala, 4, 0
   
disparando:: .db #0   ;;Lo usaremos para saber cuando estan disparando una bala
num_balas:  .db #1    ;;Numero maximo de balas que pueden estar en pantalla a la vez, decrementa cada vez que disporamos

.area _CODE

;;===================================================
;;Se encarga de llamar a todas las funciones de dibujado
;;DESTROYS: AF, BC, DE, HL
;;===================================================
bullet_update::
   ld    a, (disparando)      ;;metemos en A el valor de disparando
   cp    #0                   ;;Comparo el registro A con un 0 para saber si esta disparando
   ret   z                    ;;Si el resultado es 0, no dibujamos nada

   ld    a, (player_nivel)
   cp    #4
   jr    nz, no_aceleres
      ld    a, #8
      ld    (bullet_speed), a
   no_aceleres:
   
   call  move                 ;;movemos la bala por pantalla
   ret

;;===================================================
;;Llama a todas las funciones de dibujado
;;DESTROYS: AF, BC, DE, HL, IX
;;===================================================
bullet_render::
   ld    a, (disparando)
   cp    #0
   ret   z

   ld    ix, #bullet_data
   call  entity_erase
   call  entity_draw
   call  entity_drawUpdate
   ret

;;===================================================
;;Crea una nueva bala desde la posicion del jugador y actualiza variables (Constructor)
;;DESTROYS: AF,HL
;;===================================================
bullet_shoot::
   call  player_getPointerIX
   ld    a, (num_balas)                ;;metemos en A el numero de balas disponibles
   cp    #0                            ;;Comparo el registro A con un 0 para saber si quedan balas
   jr    z, no_balas_disponibles       ;;Si el resultado de la comparacion es 0 es que no queda balas
      ;;si quedan balas disponibles
      dec   a                          ;;decrementamos las balas disponibles
      ld    (num_balas), a             ;;actualizamos las balas que quedan

      ld    a, ent_x(ix)               ;;obtenemos la posicion x del jugador y la guardamos en el registro A
      inc   a
      ld    (bullet_x), a              ;;cambiamos la posicion x de la bala a la misma que la del jugador
      ld      a, ent_y(ix)             ;;obtenemos la posicion y del jugador y la guardamos en el registro A
      sub     a, #4                    ;;sustraemos del registro A un numero dado
      ld      (bullet_y), a            ;;|cambiamos la posicion y de la bala a la misma que la del jugador pero mas alta
      ld    a, #0xF0                   ;;|
      ld    h, ent_sprite_h(ix)
      ld    l, ent_sprite_l(ix)

      ld    a, #1                   ;;Cargamos un 1 en el registro A
      ld    (disparando), a         ;;Ponemos el booleano a 1 (true)
   no_balas_disponibles:
   ret

;;===================================================
;;Comprueba si se esta disparando una bala para dibujarla o no
;;DESTROYS: AF, B
;;===================================================
move:
   ld    ix, #bullet_data
   ld    a, (bullet_y)           ;;metemos en A el valor de bullet_y
   ld    b, ent_speed(ix)        ;;Cargamos la velocidad en B (contador del bucle)
   seguir_avanzando:
   cp    #4                      ;;Comprueba con el limite superior de la pantalla
   jr    z, borrar_bala          ;;Si estas en el limite se salta la resta y borra la bala
      dec a                      ;;A--(bullet_y--)
      ld (bullet_y), a             ;;actualizamos bullet_y
      dec b                      ;;Decrementamos el contador
      jr nz, seguir_avanzando    ;;Seguimos moviendo si aun no hemos completado los 4 pixeles
   jr    movimiento_valido       ;;Si ha podido moverse todas las  veces se salta la parte en la que borramos la bala
   borrar_bala:
   call bullet_delete
   movimiento_valido:

   ld    a, #2
   ld    (bullet_drawC), a

   ret

;;===================================================
;;"Eliminamos" una bala aumentando el contador y poniendo el disparando a false
;;DESTROYS: AF
;;===================================================
bullet_delete::
   ld    a,(num_balas)        ;;|
   inc   a                    ;;| Aumentamos el numero de balas restantes
   ld    (num_balas), a       ;;|

   ld    ix, #bullet_data
   call  entity_eraseLast

   ld    a, #0                ;;|  
   ld    (disparando), a      ;;| Volvemos a poner disparando a 0 (false)
   ld    (bullet_drawC), a
   ret

;;===================================================
;;Devuelve en A el drawC de la entidad
;;DESTROYS: AF
;;RETURNS: A = drawC
;;===================================================
bullet_getDrawC::
   ld    a, (bullet_drawC)
   ret


;;===================================================
;;Devuelve en IY un puntero al inicio de sus datos
;;DESTROYS: IY
;;RETURNS: IY = Puntero a datos bullet
;;===================================================
bullet_getPointerIY::
   ld    iy, #bullet_data
   ret