;; My Galaxy spaceship game
;; Copyright (C) 2017 AyMy Studios
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


.area _DATA

.include "headers/entity.h.s"

.globl cpct_getScreenPtr_asm
.globl cpct_drawSprite_asm
.globl cpct_drawSolidBox_asm
.globl video_getVideoPointer
.globl video_getBackVideoPointer

.globl player_vida

.area _CODE

;;===================================================
;;Dibuja el sprite de la entidad
;;INPUTS: IX => Puntero a los datos de entidad
;;DESTROYS: AF, HL
;;===================================================
entity_draw::
   ld    h, ent_sprite_h(ix)
   ld    l, ent_sprite_l(ix)
;    call render
   push  hl                      ;;Guarda A(color elgido por el usuario) en una pila
   ;;Calcula la posicion en pantalla
   call  video_getVideoPointer                 
   ld    c, ent_x(ix)            ;;|C = entity_x
   ld    b, ent_y(ix)            ;;|B = entity_y
   call  cpct_getScreenPtr_asm   ;;Coje el puntero a la pantalla y lo guarda en HL

   ;;Dibujamos un sprite
   ex    de, hl                  ;;Puntero a la posicion de memoria en DE
   pop   hl                      ;;HL = Sprite
   ld    c, ent_w(ix)
   ld    b, ent_h(ix)
   call  cpct_drawSprite_asm
   ret

;;===================================================
;;Dibuja una caja negra para borrar la entidad
;;INPUTS: IX => Puntero a los datos de entidad
;;DESTROYS: AF, BC, DE, HL
;;===================================================
entity_erase::

   ;;Calcula la posicion en pantalla
   call  video_getVideoPointer                 
   ld    c, ent_x+2(ix)          ;;|C = entity_x
   ld    b, ent_y+2(ix)          ;;|B = entity_y
   call  cpct_getScreenPtr_asm   ;;Coje el puntero a la pantalla y lo guarda en HL

   ;;Dibujamos una caja
   ex    de, hl                  ;;intercambia lo que hay en HL por lo que hay en DE
   ld    a, #0x00                ;;Guarda negro en A
   ld    c, ent_w(ix)
   ld    b, ent_h(ix)
   call  cpct_drawSolidBox_asm
   ret 

;;===================================================
;;Dibuja una caja negra en los dos ultimos frames de la entidad para evitar parpadeos
;;INPUTS: IX => Puntero a los datos de entidad
;;DESTROYS: AF, BC, DE, HL
;;===================================================
entity_eraseLast::

   ;;Calcula la posicion en pantalla
   call  video_getVideoPointer                 
   ld    c, ent_x(ix)          ;;|C = entity_x
   ld    b, ent_y(ix)          ;;|B = entity_y
   call  cpct_getScreenPtr_asm   ;;Coje el puntero a la pantalla y lo guarda en HL

   ;;Dibujamos una caja
   ex    de, hl                  ;;intercambia lo que hay en HL por lo que hay en DE
   ld    a, #0x00                ;;Guarda negro en A
   ld    c, ent_w(ix)
   ld    b, ent_h(ix)
   call  cpct_drawSolidBox_asm

   ;;Calcula la posicion en pantalla
   call  video_getVideoPointer                 
   ld    c, ent_x+1(ix)          ;;|C = entity_x
   ld    b, ent_y+1(ix)          ;;|B = entity_y
   call  cpct_getScreenPtr_asm   ;;Coje el puntero a la pantalla y lo guarda en HL

   ;;Dibujamos una caja
   ex    de, hl                  ;;intercambia lo que hay en HL por lo que hay en DE
   ld    a, #0x00                ;;Guarda negro en A
   ld    c, ent_w(ix)
   ld    b, ent_h(ix)
   call  cpct_drawSolidBox_asm

   ;;Calcula la posicion en pantalla
   call  video_getBackVideoPointer                 
   ld    c, ent_x+2(ix)          ;;|C = entity_x
   ld    b, ent_y+2(ix)          ;;|B = entity_y
   call  cpct_getScreenPtr_asm   ;;Coje el puntero a la pantalla y lo guarda en HL

   ;;Dibujamos una caja
   ex    de, hl                  ;;intercambia lo que hay en HL por lo que hay en DE
   ld    a, #0x00                ;;Guarda negro en A
   ld    c, ent_w(ix)
   ld    b, ent_h(ix)
   call  cpct_drawSolidBox_asm

   ld    a, #0
   ld    ent_drawC(ix), a
   ret 

;;===================================================
;;Actualiza valores de fotogramas para el doble buffer
;;INPUTS:
;;    IX => Puntero a los datos de entidad
;;DESTROYS: AF, BC
;;===================================================
entity_drawUpdate::
   ld    a, ent_x+1(ix)       ;;Cargamos posicion central del array de X
   ld    ent_x+2(ix), a       ;;La copiamos en la ultima posicion
   ld    a, ent_y+1(ix)       ;;Cargamos posicion central del array de Y
   ld    ent_y+2(ix), a       ;;La copiamos en la ultima posicion

   ld    a, ent_x+0(ix)       ;;Cargamos primera posicion del array de X
   ld    ent_x+1(ix), a       ;;La copiamos en la posicion central
   ld    a, ent_y+0(ix)       ;;Cargamos primera posicion del array de Y
   ld    ent_y+1(ix), a       ;;La copiamos en la posicion central

   ld    a, ent_drawC(ix)     ;;|
   dec   a                    ;;|
   ld    ent_drawC(ix), a     ;;\ Decrementamos el numero de veces que hay que pintar
   ret