;; My Galaxy spaceship game
;; Copyright (C) 2017 AyMy Studios
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


.area _DATA

.include "headers/main.h.s"

estado:: .db #0       ;;0 => Menu,   1 => Jugando,   2 => Game Over

vel_musica::  .db #4
unavariable:  .db #4   ;;6 para 50hz, 12 para 25Hz, 3 para 100Hz

.area _CODE

;;===================================================
;;Encargada de las interrupciones para la musica
;;Se asegura que la musica se llame las veces que haga falta mediante un cont
;:DESTROYS: AF, AF, BC, BC', DE, DE', HL, HL', IX, IY
;;===================================================
;isr:
;   ex    af, af'  ;;cambiamos el registro af por su alternativo
;   exx            ;;cambia los registros bc, de y hl por sus alternativos
;   push  af
;   push  bc
;   push  de 
;   push  hl
;   push  iy
;
;   ld    a, (unavariable)
;   dec   a
;   ld (unavariable), a
;   jr    nz, return
;
;      call cpct_akp_musicPlay_asm ;;como la cancion esta creada a 50Hz hay que llamarla 50 veces por segundo
;      ld    a, (vel_musica)
;      ld (unavariable), a
;
;   return:
;   pop   iy
;   pop   hl
;   pop   de
;   pop   bc
;   pop   af
;   exx
;   ex    af, af'
;   ret

;;===================================================
;;Se encarga de inicializar todo para empezar a jugar
;:DESTROYS: HL, DE, BC
;;===================================================
initialize:
   call cpct_disableFirmware_asm ;;deshabilitamos le Firware

   ld    c, #0                 ;;Ponemos en C un 0
   call  cpct_setVideoMode_asm ;;Cambiamos el modo de video al que esta en el registro C (0)

   ;;para poner la paleta de color
   ld    hl, #_sprite_palette
   ld    de, #16
   ;;esta funcion necesita la paleta en hl
   call cpct_setPalette_asm

;   ;;inicializamos la musica
;   ld    de, #_song_ingame
;   call cpct_akp_musicInit_asm
;
;   ;;llamada a la funcion isr cada vez que haya una interrupcion
;   ld    hl, #isr
;   call cpct_setInterruptHandler_asm

   ;;para meter el tileset con los tiles
   ld    hl, #_til_tileset
   ;;esta funcion necesita el tileset en hl
   call cpct_etm_setTileset2x4_asm

   ;;dibujamos el menu
   call  hud_draw_main_menu

   ret


;;===================================================
;; Programa principal
;;===================================================
_main::
   ld    sp, #0x8000
   
   ;;Set-up inicial
   call initialize

   main_loop:

   ;;Comprobamos si estamos en el menu
   ld    a, (estado)
   cp    #0
   jr    nz, no_menu

      call  menu_input
      call  menu_update

   no_menu:

   ;;Comprobamos si estamos jugando
   ld    a, (estado)
   cp    #1
   jr    nz, no_jugando

      ;;Esta controlado para que se dibujen una sola vez
      call hud_draw_marco
      call hud_draw_initial_text
   
      call  player_input
      call  player_getDrawC
      cp    #0
      jr    z, no_player_render
         call  player_render
      no_player_render:   
      
      call  bullet_update
      call  bullet_getDrawC
      cp    #0
      jr    z, no_bullet_render
         call  bullet_render
      no_bullet_render:
      
      call  enemy_update
      call  enemy_render
      
   no_jugando:

   ;;Comprobamos si estamos muertos
   ld    a, (estado)
   cp    #2
   jr   nz,  no_muerto

      call hud_draw_game_over
      call player_input

   no_muerto:

   call cpct_waitVSYNC_asm
   call video_switchBuffers

   jr main_loop
