;; My Galaxy spaceship game
;; Copyright (C) 2017 AyMy Studios
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


.area _DATA

.include "headers/keyboard.h.s"
.include "headers/cpct_keyborard.h.s"

.globl estado
.globl new_game
.globl hud_draw_char
.globl hud_draw_controls_menu
.globl hud_draw_main_menu
.globl video_getVideoPointer

opcion_sel:    .db #0      ;;{0 = EMPEZAR, 1 = CONTROLES{2 = ATRAS}}
selector:      .db #62

.area _CODE

;;===================================================
;;Muestra la pantalla de controles
;;DESTROYS: AF
;;===================================================
controles:
   ld    a, #2
   ld    (opcion_sel), a
      call hud_draw_controls_menu
   ret

;;===================================================
;;Vuelve a la pantalla principal
;;DESTROYS: AF
;;===================================================
principal:
   ld    a, #0
   ld    (opcion_sel), a
      call hud_draw_main_menu
   ret

;;===================================================
;;Entra al juego
;;DESTROYS: AF
;;===================================================
jugar:
   ld    a, #1
   ld    (estado), a
   ret

;;===================================================
;;Comprueba los inputs del usuario y realiza acciones
;;DESTROYS: AF, BC, DE, HL
;;===================================================
menu_input::
   ;;Comprueba si hay alguna tecla pulsada
   call cpct_scanKeyboard_if_asm
   ;;Si hay alguna tecla pulsada comprobamos si es una de las siguientes
   jr    z, no_key_pressed

      ;;Comprueba las teclas pulsadas
      call cpct_scanKeyboard_asm

      ;;Si estamos en la pantalla de controles, desactivamos W y S
      ld    a, (opcion_sel)
      cp    #2
      jr    z ,no_W_S

         ;;Comprueba si la tecla 'W' ha sido pulsada
         ld    hl, #Key_W              ;; HL = valor tecla W
         call  cpct_isKeyPressed_asm   ;;Comprueba que se ha pulsado la tecla W y lo guarda en el registo A y L
         jr    z, w_not_pressed        ;;Si no se ha pulsado W, no hacemos nada
            ld    a, #0                ;;|
            ld    (opcion_sel), a      ;;\Si se ha pulsado, movemos arriba
         w_not_pressed:

         ;;Comprueba si la tecla 'S' ha sido pulsada
         ld    hl, #Key_S              ;; HL = valor tecla S
         call  cpct_isKeyPressed_asm   ;;Comprueba que se ha pulsado la tecla S y lo guarda en el registo A y L
         jr    z, s_not_pressed        ;;Si no se ha pulsado S, no hacemos nada
            ld    a, #1                ;;|
            ld    (opcion_sel), a      ;;\Si se ha pulsado, movemos abajo   
         s_not_pressed:

      no_W_S:

      ;;Comprueba si la tecla 'ENTER' ha sido pulsada
      ld    hl, #Key_Return         ;;HL = valor tecla ENTER
      call  cpct_isKeyPressed_asm   ;;Comprueba que se ha pulsado la tecla R y lo guarda en el registro A y L
      jr    z, enter_not_pressed    ;;Si no se ha pulsado ENTER, no hacemos nada
         ;;Opcion 0: Empezar a jugar
         ld    a, (opcion_sel)
         cp    #0
         jr    nz, no_0_sel
            call new_game
         no_0_sel:

         ;;Opcion 1: Mostrar controles
         ld    a, (opcion_sel)
         cp    #1
         jr    nz, enter_not_pressed
            call controles

      enter_not_pressed:

      ;;Comprueba si la tecla 'ESCAPE' ha sido pulsada
         ld    hl, #Key_Esc         ;;HL = valor tecla ESCAPE
         call  cpct_isKeyPressed_asm   ;;Comprueba que se ha pulsado la tecla ESCAPE y lo guarda en el registro A y L
         jr    z, esc_not_pressed    ;;Si no se ha pulsado ESCAPE, no hacemos nada
            call  principal              ;;Volvemos al menu
         esc_not_pressed:

   no_key_pressed:
   ret

   ;;===================================================
;;Comprueba la opcion seleccionada para dibujar el selector en el lugar adecuado
;;DESTROYS: AF, BC, DE, HL
;;===================================================
menu_update::
   ld    a, (opcion_sel)
   cp    #0
   jr    nz, sel_abajo
      ;;Dibujamos el selector en Nueva Partida
      ld    a, (selector)
      ld    b, #124
      ld    c, #28
      call hud_draw_char

      ;;Dibujamos un espacio (borramos) en Controles
      ld    a, #32
      ld    b, #144
      ld    c, #28
      call  hud_draw_char
      ret
   sel_abajo:

   cp    #1
   ret   nz
      ;;Dibujamos el selector en Controles
      ld    a, (selector)
      ld    b, #144
      ld    c, #28
      call hud_draw_char

      ;;Dibujamos un espacio (borramos) en Nueva Partida
      ld    a, #32
      ld    b, #124
      ld    c, #28
      call  hud_draw_char
   ret
