.area _DATA

.include "headers/keyboard.h.s"
.include "headers/entity.h.s"
.include "headers/entity_inherited.h.s"
.include "headers/cpct_keyborard.h.s"

.globl _sprite_Nave_Fin
.globl bullet_shoot
.globl estado

.globl hud_update_lives
.globl hud_update_score
.globl hud_reset_score
.globl hud_update_level

new player 36, 150, 6, 20, _sprite_Nave_Fin, 2, 1

player_vida::     .db #3
player_score::    .db #0
player_nivel::    .db #1
siguiente_nivel:  .db #10
extra_siguiente:  .db #10

.area _CODE

;;===================================================
;;Comprueba los inputs del usuario y realiza acciones
;;DESTROYS: AF, BC, DE, HL
;;===================================================
player_input::
   ;;Comprueba si hay alguna tecla pulsada
   call cpct_scanKeyboard_if_asm
   jr    z, no_key_pressed
   ;;si hay alguna tecla pulsada comprobamos si es una de las siguientes

      ;;Comprueba las teclas pulsadas
      call cpct_scanKeyboard_asm


      ;;Comprueba si la tecla 'W' ha sido pulsada
      ld    hl, #Key_W              ;; HL = valor tecla W
      call  cpct_isKeyPressed_asm   ;;Comprueba que se ha pulsado la tecla W y lo guarda en el registo A y L
      jr    z, w_not_pressed        ;;Si no se ha pulsado W, no hacemos nada
         call movePlayerUp          ;;Si se ha pulsado, movemos arriba
      w_not_pressed:

      ;;Comprueba si la tecla 'S' ha sido pulsada
      ld    hl, #Key_S              ;; HL = valor tecla S
      call  cpct_isKeyPressed_asm   ;;Comprueba que se ha pulsado la tecla S y lo guarda en el registo A y L
      jr    z, s_not_pressed        ;;Si no se ha pulsado S, no hacemos nada
         call movePlayerDown        ;;Si se ha pulsado, movemos abajo   
      s_not_pressed:

      ;;Comprueba si la tecla 'A' ha sido pulsada
      ld    hl, #Key_A              ;;HL = valor tecla A
      call  cpct_isKeyPressed_asm   ;;Comprueba que se ha pulsado la tecla A y lo guarda en el registo A y L
      jr    z, a_not_pressed        ;;Si no se ha pulsado A, no hacemos nada
         call movePlayerLeft        ;;Si se ha pulsado, movemos a la izquierda   
      a_not_pressed:

      ;;Comprueba si la tecla 'D' ha sido pulsada
      ld    hl, #Key_D              ;; HL = valor tecla D
      call  cpct_isKeyPressed_asm   ;;Comprueba que se ha pulsado la tecla D y lo guarda en el registo A y L
      jr    z, d_not_pressed        ;;Si no se ha pulsado D, no hacemos nada
         call movePlayerRight       ;;Si se ha pulsado, movemos a la derecha     
      d_not_pressed:

      ;;Comprueba si la tecla 'Espacio' ha sido pulsada
      ld    hl, #Key_Space          ;;HL = valor tecla Espacio
      call  cpct_isKeyPressed_asm   ;;Comprueba que se ha pulsado la tecla Espacio y lo guarda en el registro A y L
      jr    z, space_not_pressed    ;;Si no se ha pulsado Espacio, no hacemos nada
         call bullet_shoot          ;;Si se ha pulsado, disparamos
         ;;ld    a, #2                ;;|
         ;;ld    (player_drawC), a    ;;\Le indicamos las veces que debe dibujarse
      space_not_pressed:

      ;;Para cuando morimos
      ld    a, (estado)
      cp    #2
      jr    nz, no_muerto
         
         ;;Comprueba si la tecla 'ENTER' ha sido pulsada
         ld    hl, #Key_Return         ;;HL = valor tecla ENTER
         call  cpct_isKeyPressed_asm   ;;Comprueba que se ha pulsado la tecla ENTER y lo guarda en el registro A y L
         jr    z, enter_not_pressed    ;;Si no se ha pulsado ENTER, no hacemos nada
            call new_game              ;;Iniciamos un juego nuevo
         enter_not_pressed:

         ;;Comprueba si la tecla 'ESCAPE' ha sido pulsada
         ld    hl, #Key_Esc         ;;HL = valor tecla ESCAPE
         call  cpct_isKeyPressed_asm   ;;Comprueba que se ha pulsado la tecla ESCAPE y lo guarda en el registro A y L
         jr    z, esc_not_pressed    ;;Si no se ha pulsado ESCAPE, no hacemos nada
            call  menu              ;;Volvemos al menu
         esc_not_pressed:

      no_muerto:
   no_key_pressed:
   ret

;;===================================================
;;Llama a todas las funciones de dibujado
;;DESTROYS: AF, BC, DE, HL, IX
;;===================================================
player_render::
   ld    ix, #player_data
   call  entity_erase
   call  entity_draw
   call  entity_drawUpdate
   ret

;;===================================================
;;Mueve el jugador hacia arriba si no ha llegado al limite
;;DESTROYS: AF, BC
;;===================================================
movePlayerUp:
   ld    a, (player_y)              ;;|
   ld    b, #2                      ;;|
   sub   b                          ;;\ Restamos 2 a la posicion
   cp    #15                        ;;|
   jr    nc, no_limit_up            ;;|
      ld    a, #15                  ;;|
   no_limit_up:                     ;;|
   ld    (player_y), a              ;;\Si pasa el limite, lo ponemos en el limite, si no, en su posicion

   ld    a, #2                      ;;|
   ld    (player_drawC), a          ;;\Le indicamos las veces que debe dibujarse

   ret


;;===================================================
;;Mueve el jugador hacia abajo si no ha llegado al limite
;;DESTROYS: AF, BC
;;===================================================
movePlayerDown:
   ld    ix, #player_data           ;;Cargamos en IX los datos del jugador
   ld    a, (player_y)              ;;|
   ld    b, #2                      ;;|
   add   b                          ;;\ Sumamos 2 a la posicion
   cp    ent_max_y(ix)              ;;|
   jr    c, no_limit_down           ;;|
      ld    a, ent_max_y(ix)        ;;|
   no_limit_down:                   ;;|
   ld    (player_y), a              ;;\Si pasa el limite, lo ponemos en el limite, si no, en su posicion

   ld    a, #2                      ;;|
   ld    (player_drawC), a          ;;\Le indicamos las veces que debe dibujarse

   ret


;;===================================================
;;Mueve el jugador a la izquierda si no ha llegado al limite
;;DESTROYS: AF, BC
;;===================================================
movePlayerLeft:
   ld    a, (player_x)              ;;|
   ld    b, #1                      ;;|
   sub   b                          ;;\ Restamos 1 a la posicion
   cp    #4                         ;;|
   jr    nc, no_limit_left          ;;|
      ld    a, #4                   ;;|
   no_limit_left:                   ;;|
   ld    (player_x), a              ;;\Si pasa el limite, lo ponemos en el limite, si no, en su posicion

   ld    a, #2                      ;;|
   ld    (player_drawC), a          ;;\Le indicamos las veces que debe dibujarse

   ret


;;===================================================
;;Mueve el jugador a la derecha si no ha llegado al limite
;;DESTROYS: AF, BC
;;===================================================
movePlayerRight:
   ld    ix, #player_data           ;;Cargamos en IX los datos del jugador
   ld    a, (player_x)              ;;|
   ld    b, #1                      ;;|
   add   b                          ;;\ Sumamos 1 a la posicion
   cp    ent_max_x(ix)              ;;|
   jr    c, no_limit_right          ;;|
      ld    a, ent_max_x(ix)        ;;|
   no_limit_right:                  ;;|
   ld    (player_x), a              ;;\Si pasa el limite, lo ponemos en el limite, si no, en su posicion

   ld    a, #2                      ;;|
   ld    (player_drawC), a          ;;\Le indicamos las veces que debe dibujarse

   ret

;;===================================================
;;Va al menu principal
;;DESTROYS: AF
;;===================================================
menu:
   ld    a, #0
   ld    (estado), a
   ret


;;===================================================
;;Aumenta la puntuacion
;;DESTROYS: AF
;;===================================================
player_kill::
   ;;Aumentar puntuacion
   ld    a, (player_score)
   inc   a
   ld    (player_score), a
   call  hud_update_score

   ld    a, (player_nivel)
   cp    #6
   ret   z

   ;;Comprobar para subir nivel
   ld    a, (siguiente_nivel)
   dec   a
   ld    (siguiente_nivel), a
   cp    #0
   ret   nz
      call player_1Up
      ;;Subir nivel
      ld    a, (player_nivel)
      inc   a
      ld    (player_nivel), a
      call  hud_update_level

      ld    a, (extra_siguiente)
      add   #5
      ld    (extra_siguiente), a
      ld    (siguiente_nivel), a
   ret

;;===================================================
;;Resta vida al personaje
;;DESTROYS: AF
;;===================================================
player_1Up::
   ld    a, (player_vida)   
   inc   a
   
   cp #10
   ret   z
   ;;Actualizar contador de vidas   
   push  af
   call  hud_update_lives
   pop   af
   ld    (player_vida), a
   ret

;;===================================================
;;Resta vida al personaje
;;DESTROYS: AF
;;===================================================
player_hit::
   ld    a, (player_vida)   
   dec   a
   
   ;;Actualizar contador de vidas   
   push  af
   call  hud_update_lives

   pop   af
   ld    (player_vida), a
   cp    #0
   jr    nz, no_matar
      ld    a, #2
      ld    (estado), a
   no_matar:

   ;;Mover a posicion inicial
   ld    a, #29
   ld    (player_x), a
   ld    a, #150
   ld    (player_y), a

   ld    a, #2
   ld    (player_drawC), a

   ret

;;===================================================
;;Comienza un nuevo juego
;;DESTROYS: AF
;;===================================================
new_game::
   ld    a, #3
   ld    (player_vida), a
   call  hud_update_lives

   ld    a, #0
   ld    (player_score), a
   call  hud_reset_score

   ld    a, #1
   ld    (player_nivel), a
   ld    a, #10
   ld    (siguiente_nivel), a
   ld    (extra_siguiente), a 

   ld    a, #29
   ld    (player_x), a

   ld    a, #150
   ld    (player_y), a

   call bullet_getPointerIY
   ld    a, #4
   ld    ent_speed(iy), a

   ld    a, #1
   ld    (estado), a

   ret

;;===================================================
;;Devuelve en A el drawC de la entidad
;;DESTROYS: AF
;;RETURNS: A = drawC
;;===================================================
player_getDrawC::
   ld    a, (player_drawC)
   ret


;;===================================================
;;Devuelve en IX un puntero al inicio de sus datos
;;DESTROYS: IX
;;RETURNS: IX = Puntero a datos player
;;===================================================
player_getPointerIX::
   ld    ix, #player_data
   ret

;;===================================================
;;Devuelve en IY un puntero al inicio de sus datos
;;DESTROYS: IY
;;RETURNS: IY = Puntero a datos player
;;===================================================
player_getPointerIY::
   ld    iy, #player_data
   ret